@@ -2,7 +2,7 @@ import asyncio
 import logging
 from typing import List, Optional
 
-from examples.tutorial.step4.models import Message, encode_dataclass, decode_dataclass
+from examples.tutorial.step4.shared import Message, encode_dataclass, decode_dataclass
 from reactivestreams.subscriber import DefaultSubscriber
 from reactivestreams.subscription import DefaultSubscription
 from rsocket.awaitable.awaitable_rsocket import AwaitableRSocket
@@ -77,6 +77,11 @@ class ChatClient:
         response = await AwaitableRSocket(self._rsocket).request_stream(request)
         return list(map(lambda _: utf8_decode(_.data), response))
 
+    async def list_channel_users(self, channel_name: str):
+        request = Payload(ensure_bytes(channel_name), composite(route('channel.users')))
+        response = await AwaitableRSocket(self._rsocket).request_stream(request)
+        return list(map(lambda _: utf8_decode(_.data), response))
+
 
 async def main():
     connection1 = await asyncio.open_connection('localhost', 6565)
@@ -100,16 +105,22 @@ async def messaging_example(user1: ChatClient, user2: ChatClient):
     user1.listen_for_messages()
     user2.listen_for_messages()
 
-    await user1.join('channel1')
-    await user2.join('channel1')
+    channel_name = 'channel1'
+
+    await user1.join(channel_name)
+    await user2.join(channel_name)
 
     print(f'Channels: {await user1.list_channels()}')
+    print(f'Channel {channel_name} users: {await user1.list_channel_users(channel_name)}')
 
     await user1.private_message('user2', 'private message from user1')
-    await user1.channel_message('channel1', 'channel message from user1')
+    await user1.channel_message(channel_name, 'channel message from user1')
 
     await asyncio.sleep(1)
 
+    await user1.leave(channel_name)
+    print(f'Channel {channel_name} users: {await user1.list_channel_users(channel_name)}')
+
     user1.stop_listening_for_messages()
     user2.stop_listening_for_messages()
 
