@@ -8,6 +8,7 @@ import pytest
 from performance.performance_client import PerformanceClient
 from performance.performance_server import run_server
 from rsocket.rsocket_server import RSocketServer
+from tests.tools.helpers import measure_time
 
 
 @pytest.mark.timeout(5)
@@ -31,6 +32,13 @@ async def test_request_stream(unused_tcp_port):
         assert result is not None
 
 
+@pytest.mark.performance
+async def test_large_request():
+    async with run_with_client(6565) as client:
+        result = await measure_time(client.large_request())
+        print(result.delta)
+
+
 @asynccontextmanager
 async def run_against_server(unused_tcp_port: int) -> PerformanceClient:
     server_ready = asyncio.Event()
