@@ -1,8 +1,10 @@
+import asyncio
 from typing import List, Optional
 
 from reactivestreams.publisher import Publisher
 from rsocket.awaitable.collector_subscriber import CollectorSubscriber
 from rsocket.frame import MAX_REQUEST_N
+from rsocket.local_typing import Awaitable
 from rsocket.payload import Payload
 from rsocket.rsocket import RSocket
 
@@ -12,31 +14,34 @@ class AwaitableRSocket:
     def __init__(self, rsocket: RSocket):
         self._rsocket = rsocket
 
-    def fire_and_forget(self, payload: Payload):
-        self._rsocket.fire_and_forget(payload)
+    def fire_and_forget(self, payload: Payload) -> Awaitable[None]:
+        return self._rsocket.fire_and_forget(payload)
 
-    def metadata_push(self, metadata: bytes):
-        self._rsocket.metadata_push(metadata)
+    def metadata_push(self, metadata: bytes) -> Awaitable[None]:
+        return self._rsocket.metadata_push(metadata)
 
     async def request_response(self, payload: Payload) -> Payload:
         return await self._rsocket.request_response(payload)
 
     async def request_stream(self,
                              payload: Payload,
-                             initial_request_n=MAX_REQUEST_N) -> List[Payload]:
-        subscriber = CollectorSubscriber()
+                             limit_rate=MAX_REQUEST_N) -> List[Payload]:
+        subscriber = CollectorSubscriber(limit_rate)
 
-        self._rsocket.request_stream(payload).initial_request_n(initial_request_n).subscribe(subscriber)
+        self._rsocket.request_stream(payload).initial_request_n(limit_rate).subscribe(subscriber)
 
         return await subscriber.run()
 
     async def request_channel(self,
                               payload: Payload,
                               publisher: Optional[Publisher] = None,
-                              initial_request_n=MAX_REQUEST_N) -> List[Payload]:
-        subscriber = CollectorSubscriber()
+                              limit_rate=MAX_REQUEST_N,
+                              sending_done: Optional[asyncio.Event] = None) -> List[Payload]:
+        subscriber = CollectorSubscriber(limit_rate)
 
-        self._rsocket.request_channel(payload, publisher).initial_request_n(initial_request_n).subscribe(subscriber)
+        self._rsocket.request_channel(payload,
+                                      publisher=publisher,
+                                      sending_done=sending_done).initial_request_n(limit_rate).subscribe(subscriber)
 
         return await subscriber.run()
 
