@@ -1,16 +1,16 @@
 import asyncio
 from asyncio import Future
-
-from typing import Optional, cast
+from typing import Optional, cast, Union, Callable
 
 import reactivex
-from reactivex import Observable
+from reactivex import Observable, Subject, operators
 
 from rsocket.frame import MAX_REQUEST_N
+from rsocket.helpers import is_non_empty_payload
 from rsocket.payload import Payload
-from rsocket.rsocket import RSocket
-from rsocket.reactivex.back_pressure_publisher import BackPressurePublisher
+from rsocket.reactivex.back_pressure_publisher import observable_to_publisher
 from rsocket.reactivex.from_rsocket_publisher import from_rsocket_publisher
+from rsocket.rsocket import RSocket
 
 
 class ReactiveXClient:
@@ -22,20 +22,19 @@ class ReactiveXClient:
         return from_rsocket_publisher(response_publisher, request_limit)
 
     def request_response(self, request: Payload) -> Observable:
-        return reactivex.from_future(cast(Future, self._rsocket.request_response(request)))
+        return reactivex.from_future(cast(Future, self._rsocket.request_response(request))).pipe(
+            operators.filter(is_non_empty_payload)
+        )
 
     def request_channel(self,
                         request: Payload,
                         request_limit: int = MAX_REQUEST_N,
-                        observable: Optional[Observable] = None,
-                        sending_done_event: Optional[asyncio.Event] = None) -> Observable:
-        if observable is not None:
-            local_publisher = BackPressurePublisher(observable)
-        else:
-            local_publisher = None
+                        observable: Optional[Union[Observable, Callable[[Subject], Observable]]] = None,
+                        sending_done: Optional[asyncio.Event] = None) -> Observable:
+        requester_publisher = observable_to_publisher(observable)
 
         response_publisher = self._rsocket.request_channel(
-            request, local_publisher, sending_done_event
+            request, requester_publisher, sending_done
         ).initial_request_n(request_limit)
         return from_rsocket_publisher(response_publisher, request_limit)
 
